--Fuzzy Logic Library
--9-28-98
--Hawke' <mdeland@nwinfo.net> <trumpyt@nwinfo.net>
--This library is copyright reserved shareware.
--See Fuzzy.doc for usage, version and shareware information.

constant LOWER=1,
	 UPPER=2

function IsSame(object a, object b)
   return compare(a,b)=0
end function

procedure Error(sequence msg, object x)
atom choice,fn
integer done
   puts(1,"Error Detected...\n")
   puts(1,msg & "\n\n")
   puts(1,"Do you want a logfile? <Y>es, <N>o\n")
   done = 0
   while not done do
        choice = get_key()
        if  find(choice,"Yy") then
            puts(1,"Attempting to save data...\n")
            fn = open("Fuzzy.err","w")
            if fn=-1 then
                puts(1,"Cannot open Fuzzy.err\n")
                abort(1)
            else
               if atom(x) then
                    printf(fn,"\nThis is the data:\n%g\n",x)
               else for i=1 to length(x) do
                      printf(fn,"\nThis is object #%d:\n",i)
                      print(fn,x[i])
                    end for
               end if
            end if
            close(fn)
            done=1
        elsif find(choice,"Nn") then done = 1
	end if
   end while
   puts(1,"The data should be in Fuzzy.err now.\n")
   abort (1)
end procedure

function IsAllAtoms(object x)
   for i=1 to length(x) do
      if sequence(x[i]) then return 0 end if
   end for
   return 1
end function

procedure CheckMismatch(object a, object b, sequence caller)
   if ( sequence(a) and atom(b) ) or
      ( atom(a) and sequence(b) ) then
      Error(caller & ":Mismatched objects found.",{a,b})
   end if
end procedure

procedure CheckLengths(sequence a, sequence b, sequence caller)
   if length(a) != length(b) then
      Error(caller & ":Unequal length sequences.",{a,b})
   end if
end procedure

procedure CheckTruths(object truth)
object lwr,upr
   if atom(truth) then
      Error("A truth table was a singular atom.",truth)
   end if
   if length(truth)!=2 then
      Error("A truth table was an incorrect length.",truth)
   end if
   lwr=truth[LOWER]  upr=truth[UPPER]
   CheckMismatch(lwr,upr,"CheckTruths")
   if ( sequence(lwr) and sequence(upr) ) then
      CheckLengths(lwr,upr,"CheckTruths")
      for i=1 to length(lwr) do
          if sequence(lwr[i]) or sequence(upr[i]) then
             Error("CheckTruths:Sequences nested too deeply.",truth)
          end if
      end for
   end if
end procedure

procedure CheckObjects(object x, object y, sequence caller)
   CheckMismatch(x,y,caller)
   if sequence(x) then --and sequence(y), but that is already done
      CheckLengths (x,y,caller)
      for i=1 to length(x) do
          CheckObjects(x[i],y[i],caller)
      end for
   end if
end procedure

function IsTruthAtom(object truth)
   return atom(truth[1])
end function

function Minimum(object a, object b)
object ai,bi
   if atom(a) and atom(b) then
      if b<a then a=b end if   return a
   else for i=1 to length(a) do
           ai=a[i]  bi=b[i]
           if atom(ai) then
                if bi<ai then ai=bi end if
           else ai=Minimum(ai,bi)
           end if
           a[i]=ai
        end for
   end if
   return a
end function

function Maximum(object a, object b)
object ai,bi
   if atom(a) and atom(b) then
      if a<b then a=b end if   return a
   else for i=1 to length(a) do
           ai=a[i]  bi=b[i]
           if atom(ai) then
                if ai<bi then ai=bi end if
           else ai=Maximum(ai,bi)
           end if
           a[i]=ai
        end for
   end if
   return a
end function

global function FindTruthAtom(object x, sequence truths)
atom divdr,lwr,upr
object tmp
   if atom(x) then
      if x<=truths[LOWER] then return 0 end if
      if x>=truths[UPPER] then return 1 end if
      return (x-truths[LOWER]) /  (truths[UPPER]-truths[LOWER])
   else
        lwr  =truths[LOWER]
        upr  =truths[UPPER]
        divdr=upr-lwr
	for i=1 to length(x) do
           tmp=x[i]
           if atom(tmp) then
                if    tmp<=lwr then tmp=0
                elsif tmp>=upr then tmp=1
                else  tmp=(tmp-lwr) / divdr
                end if
           else tmp=FindTruthAtom(tmp,truths)
           end if
           x[i]=tmp
	end for
   end if
   return x
end function

function CalcSeqvsSeq(sequence x,sequence truths)
object lwr,upr,divdr,tmp
   for i=1 to length(x) do
        lwr=truths[LOWER][i]
        upr=truths[UPPER][i]
        divdr=upr-lwr
        tmp=x[i]
        if    tmp<=lwr then tmp=0
        elsif tmp>=upr then tmp=1
        else  tmp=(tmp-lwr) / divdr
        end if
        x[i]=tmp
   end for
   return x
end function

function FindTruthSeqE(object x, sequence truths)
--SeqE=_Seq_uence with _E_rror checking
--called from FindTruth() generic
   if atom(x) then
      Error("FindTruthSeq:x was improperly nested.",{x,truths})
   end if
   if IsAllAtoms(x) then
      CheckLengths(x,truths[LOWER],"FindTruthSeq")
      return CalcSeqvsSeq(x,truths)
   else  for i=1 to length(x) do
             x[i]=FindTruthSeqE(x[i],truths)
         end for
   end if
   return x
end function

global function FindTruthSeq(object x, sequence truths)
object tmp
   if IsAllAtoms(x) then
      return CalcSeqvsSeq(x,truths)
   else  for i=1 to length(x) do
            tmp=x[i]
            if IsAllAtoms(tmp) then
                 tmp=CalcSeqvsSeq(tmp,truths)
            else tmp=FindTruthSeq(tmp,truths)
            end if
            x[i]=tmp
         end for
   end if
   return x
end function

global function FindTruth(object x, sequence truths)
   CheckTruths(truths)         --is truths valid?
   if IsTruthAtom(truths) then --is truths {atom,atom}?
        return FindTruthAtom(x,truths) --x,{atom,atom}
   else return FindTruthSeqE(x,truths) --x,{seq ,seq}
   end if
end function

global function FuzzyNOT(object x, sequence truths)
   if IsSame(truths,{}) then return 1.0-x  end if
   return 1.0-FindTruth(x,truths)
end function

global function FuzzyNOTAtom(object x, sequence truths)
   if IsSame(truths,{}) then return 1.0-x  end if
   return 1.0-FindTruthAtom(x,truths)
end function

global function FuzzyNOTSeq(object x, sequence truths)
   if IsSame(truths,{}) then return 1.0-x  end if
   return 1.0-FindTruthSeq(x,truths)
end function

global function FuzzyAND
(object x,sequence truthX,object y,sequence truthY)
   CheckObjects(x,y,"FuzzyAND")
   if not IsSame(truthX,{}) then x=FindTruth(x,truthX) end if
   if not IsSame(truthY,{}) then y=FindTruth(y,truthY) end if
   return Minimum(x,y) 
end function

global function FuzzyANDAtom
(object x,sequence truthX,object y,sequence truthY)
   if not IsSame(truthX,{}) then x=FindTruthAtom(x,truthX) end if
   if not IsSame(truthY,{}) then y=FindTruthAtom(y,truthY) end if
   return Minimum(x,y) 
end function

global function FuzzyANDSeq
(object x,sequence truthX,object y,sequence truthY)
   if not IsSame(truthX,{}) then x=FindTruthSeq(x,truthX) end if
   if not IsSame(truthY,{}) then y=FindTruthSeq(y,truthY) end if
   return Minimum(x,y) 
end function

global function FuzzyOR
(object x,sequence truthX,object y,sequence truthY)
   CheckObjects(x,y,"FuzzyOR")
   if not IsSame(truthX,{}) then x=FindTruth(x,truthX) end if
   if not IsSame(truthY,{}) then y=FindTruth(y,truthY) end if
   return Maximum(x,y)
end function

global function FuzzyORAtom
(object x,sequence truthX,object y,sequence truthY)
   if not IsSame(truthX,{}) then x=FindTruthAtom(x,truthX) end if
   if not IsSame(truthY,{}) then y=FindTruthAtom(y,truthY) end if
   return Maximum(x,y)
end function

global function FuzzyORSeq
(object x,sequence truthX,object y,sequence truthY)
   if not IsSame(truthX,{}) then x=FindTruthSeq(x,truthX) end if
   if not IsSame(truthY,{}) then y=FindTruthSeq(y,truthY) end if
   return Maximum(x,y)
end function
