include fuzzy.e
include get.e

constant none = -9999
constant blank="        "

constant OLD   = {18,65}
--if you're under 18, you are definitly not old (not adult)
--if you're 65, you are certainly old (senior citizen)
--every other age is "in between that" and gets 0..1 rating

constant TALL  = {4,7} --feet
--if you're under 4ft, you are short for sure
--if you're over 7ft, you are assuredly tall
--all heights between 4..7 feet get 0..1 rating

constant LOUD  = {60,100} --db (decibel)
--under 60db is *quiet* actually (60-70db is the standard
--   sound level in a country home at "rest")
--over 100db is considered officially loud (jets are
--   about 110-120db, concerts are 105-145db)
--between 60..100db is average everyday soundlevels
--   so those db ratings get 0..1 ratings

constant HEAVY = {100,300} --person weight in pounds
--under 100lbs is a lightweight person
--over 300lbs is a heavy person
--between 100..300lbs is the 0..1 rating 

constant WHITE = { {190,190,190},{240,240,240} }
--truecolor RGB values 
--under {190,190,190} is not white (grey really)
--over {240,240,240} is white
--between {190,190,190} and {240,240,240} are varying
--   shades of white getting 0..1 ratings for each of
--   the RGB elements
--   (ergo:{0,0,0} is *not* white at all,
--         {0.4,0,0.7} is a little white,
--         {0.7,0.92,0.81} is a lot white,
--         {1,1,1} is a winner

--------------------------------------------------------------
function RealRand(atom limit)
   return rand(limit) + (rand(10000)/10000)
end function

procedure PressKey()
   position (23,1)
   puts(1,"Press a key")
   if wait_key() then end if
end procedure

procedure Pause(sequence hdr)
   PressKey()
   clear_screen()
   position(1,1)
   puts(1,hdr)
   position(2,1)
end procedure

function IsAllAtoms(object z)
   for i=1 to length(z) do
      if sequence(z[i]) then return 0 end if
   end for
   return 1
end function

procedure PutNum(atom a, integer CR)
   if integer(a) then printf(1,"%15d",a)
   else printf(1,"%15g",a)
   end if
   if CR then puts(1,"\n") end if
end procedure

procedure PutChar(atom a, integer CR)
   printf(1,"%15s",a)
   if CR then puts(1,"\n") end if
end procedure

procedure PutAtom(atom a,atom b,atom c)
   PutNum(a,0)
   if c=none then PutNum(b,1)
   else PutNum(b,0) PutNum(c,1)
   end if
   --this is here in case you wanted to make alphanumeric
   --if a<0 or a>255 then PutNum(a,1) --ain't ascii
   --else
   --   if integer(a) then PutChar(a,0) end if
   --   PutNum(a,1)
   --end if
end procedure

procedure PagePut(object a,object b,object c,sequence hdr)
integer line
   position (1,1)
   puts(1,hdr)
   if atom(a) then
        PutAtom(a,b,c)
   else --a is a sequence
      if IsAllAtoms(a) then --is it a single dimension?
         line = 2
         for i=1 to length(a) do
            position (line,1)
            if atom(c) then PutAtom(a[i],b[i],c)
            else PutAtom(a[i],b[i],c[i]) end if
            line=line+1
            if line>20 then Pause(hdr) line=2 end if
         end for
      else --nope, multidimension, let's recurse
         line = 1
         for i=1 to length(a) do
            if atom(c) then PagePut(a[i],b[i],c,hdr)
            else PagePut(a[i],b[i],c[i],hdr) end if
            line=line+1
            if line>20 then Pause(hdr) line=2 end if
         end for
      end if --IsAllAtoms(a)
   end if --atom(a)
   PressKey()
   clear_screen()
end procedure

-----------------------------------------------------------
--lets create some test data
sequence AgeData, HeightData, WeightData,
         SoundData, ColorData
sequence result, header
AgeData  ={} HeightData={} WeightData={} 
SoundData={} ColorData ={} 

for loop = 1 to 30 do
	AgeData   = AgeData    & RealRand(100)
        HeightData= HeightData & RealRand(9)
	WeightData= WeightData & RealRand(500)
        SoundData = SoundData  & RealRand(130)
        ColorData = append( ColorData,rand({255,255,255}) )
end for

--now we can do weird things :)
clear_screen()

position(22,1)
puts(1,"Who is 'old' relatively speaking?\n")
header = blank & "Ages   " & blank & "result"
result = FindTruth(AgeData,OLD) 
PagePut(AgeData,result,none,header)

position(22,1)
puts(1,"Who is 'young'?\n")
header = blank & "Ages   " & blank & "result"
result = FuzzyNOT(AgeData,OLD)  
PagePut(AgeData,result,none,header)

position(22,1)
puts(1,"Who is 'old' and kinda pudgy?\n")
header = blank & "Ages   " & blank & "Weights" & blank & "result"
result = FuzzyAND(AgeData,OLD,WeightData,HEAVY)
PagePut(AgeData,WeightData,result,header)

position(22,1)
puts(1,"Who is 'old' and 'short'?\n")
header = blank & "Ages   " & blank & "Heights" & blank & "result"
result = {{},{}}
result[1] = FuzzyNOT(HeightData,TALL)
result[2] = FindTruth(AgeData,OLD)
result = FuzzyAND(result[1],{},result[2],{})
PagePut(AgeData,HeightData,result,header)

position(22,1)
puts(1,"Who is 'big' or 'tall'?\n")
header = blank & "Weights" & blank & "Heights" & blank & "result"
result = FuzzyOR(WeightData,HEAVY,HeightData,TALL)
PagePut(WeightData,HeightData,result,header)

position(22,1)
puts(1,"Where can I take a nap?\n")
header = blank & "Sounds " & blank & "result"
result = FuzzyNOT(SoundData,LOUD)
PagePut(SoundData,result,none,header)

position(21,1)
puts(1,"(We're going to have to do this one 'element' at a time)")
position(22,1)
puts(1,"Which of these shoes goes with this white dress?\n")
header = blank & "Colors " & blank & "result"
result = FindTruth(ColorData,WHITE)
for i=1 to length(ColorData) do
PagePut(ColorData[i],result[i],none,header)
end for


